@@ -125,7 +125,7 @@ static const IndicOTCharClass bengCharClasses[] =
     _dr, _db, _db, _db, _db, _xx, _xx, _dl, _dl, _xx, _xx, _s1, _s2, _vr, _xx, _xx, /* 09C0 - 09CF */
     _xx, _xx, _xx, _xx, _xx, _xx, _xx, _dr, _xx, _xx, _xx, _xx, _cn, _cn, _xx, _cn, /* 09D0 - 09DF */
     _iv, _iv, _dv, _dv, _xx, _xx, _xx, _xx, _xx, _xx, _xx, _xx, _xx, _xx, _xx, _xx, /* 09E0 - 09EF */
-    _ct, _ct, _xx, _xx, _xx, _xx, _xx, _xx, _xx, _xx, _xx                           /* 09F0 - 09FA */
+    _rv, _ct, _xx, _xx, _xx, _xx, _xx, _xx, _xx, _xx, _xx                           /* 09F0 - 09FA */
 };
 
 static const IndicOTCharClass guruCharClasses[] =
@@ -181,7 +181,7 @@ static const IndicOTCharClass teluCharClasses[] =
     _xx, _mp, _mp, _mp, _xx, _iv, _iv, _iv, _iv, _iv, _iv, _iv, _iv, _xx, _iv, _iv, /* 0C00 - 0C0F */
     _iv, _xx, _iv, _iv, _iv, _bb, _bb, _bb, _bb, _bb, _bb, _bb, _bb, _bb, _bb, _bb, /* 0C10 - 0C1F */
     _bb, _bb, _bb, _bb, _bb, _bb, _bb, _bb, _bb, _xx, _bb, _bb, _bb, _bb, _bb, _bb, /* 0C20 - 0C2F */
-    _bb, _ct, _bb, _bb, _xx, _bb, _bb, _bb, _bb, _bb, _xx, _xx, _xx, _xx, _da, _da, /* 0C30 - 0C3F */
+    _bb, _bb, _bb, _bb, _xx, _bb, _bb, _bb, _bb, _bb, _xx, _xx, _xx, _xx, _da, _da, /* 0C30 - 0C3F */
     _da, _dr, _dr, _dr, _dr, _xx, _da, _da, _s1, _xx, _da, _da, _da, _vr, _xx, _xx, /* 0C40 - 0C4F */
     _xx, _xx, _xx, _xx, _xx, _da, _db, _xx, _xx, _xx, _xx, _xx, _xx, _xx, _xx, _xx, /* 0C50 - 0C5F */
     _iv, _iv, _xx, _xx, _xx, _xx, _xx, _xx, _xx, _xx, _xx, _xx, _xx, _xx, _xx, _xx  /* 0C60 - 0C6F */
@@ -450,7 +450,7 @@ static const gint8 stateTable[][CC_COUNT] =
     {-1,  6,  1, -1, -1, -1, -1,  5,  4, -1, -1},
     {-1,  6,  1, -1, -1, -1,  2,  5,  4, 10,  9},
     {-1, -1, -1, -1,  3,  2, -1, -1, -1,  8, -1},
-    {-1,  6,  1, -1, -1, -1, -1,  5, -1, -1, -1},
+    {-1,  6,  1, -1, -1, -1, -1, -1, -1, -1, -1},
     {-1,  7,  1, -1, -1, -1, -1, -1, -1, -1, -1},
     {-1, -1,  1, -1, -1, -1, -1, -1, -1, -1, -1},
     {-1, -1, -1, -1,  3,  2, -1, -1, -1, -1, -1},
@@ -469,6 +469,48 @@ glong indic_ot_find_syllable(const IndicOTClassTable *class_table, const gunicha
 
         state = stateTable[state][char_class & CF_CLASS_MASK];
 
+        /*for the components of split matra*/	
+	if ((char_count >= cursor + 3) &&
+	    (chars[cursor] == 0x0DD9 && chars[cursor + 1] == 0x0DCF && chars[cursor + 2] == 0x0DCA)) {  /*for 3 split matra of Sinhala*/
+	    return cursor + 3;
+	}        
+	else if ((char_count >= cursor + 3) &&
+	         (chars[cursor] == 0x0CC6 && chars[cursor + 1] == 0x0CC2 && chars[cursor + 2] == 0x0CD5)) {  /*for 3 split matra of Kannada*/
+	    return cursor + 3;
+	}
+        /*for 2 split matra*/	
+	else if (char_count >= cursor + 2) {
+	        /*for Bengali*/
+            if ((chars[cursor] == 0x09C7 && chars[cursor + 1] == 0x09BE) ||	       
+	        (chars[cursor] == 0x09C7 && chars[cursor + 1] == 0x09D7) ||		
+	        /*for Oriya*/
+	        (chars[cursor] == 0x0B47 && chars[cursor + 1] == 0x0B3E) ||		
+	        (chars[cursor] == 0x0B47 && chars[cursor + 1] == 0x0B56) ||		
+	        (chars[cursor] == 0x0B47 && chars[cursor + 1] == 0x0B57) ||
+	        /*for Tamil*/
+	        (chars[cursor] == 0x0BC6 && chars[cursor + 1] == 0x0BBE) ||		
+	        (chars[cursor] == 0x0BC6 && chars[cursor + 1] == 0x0BD7) ||		
+	        (chars[cursor] == 0x0BC7 && chars[cursor + 1] == 0x0BBE) ||		
+	        /*for Malayalam*/
+	        (chars[cursor] == 0x0D46 && chars[cursor + 1] == 0x0D3E) ||	
+	        (chars[cursor] == 0x0D46 && chars[cursor + 1] == 0x0D57) ||		
+	        (chars[cursor] == 0x0D47 && chars[cursor + 1] == 0x0D3E) ||	
+	        /*for Sinhala*/
+	        (chars[cursor] == 0x0DD9 && chars[cursor + 1] == 0x0DCA) ||		
+	        (chars[cursor] == 0x0DD9 && chars[cursor + 1] == 0x0DCF) ||		
+	        (chars[cursor] == 0x0DD9 && chars[cursor + 1] == 0x0DDF) ||		
+	        (chars[cursor] == 0x0DDC && chars[cursor + 1] == 0x0DCA) ||		
+	        /*for Telugu*/
+	        (chars[cursor] == 0x0C46 && chars[cursor + 1] == 0x0C56) ||	
+	        /*for Kannada*/
+	        (chars[cursor] == 0x0CBF && chars[cursor + 1] == 0x0CD5) ||		    
+	        (chars[cursor] == 0x0CC6 && chars[cursor + 1] == 0x0CD5) ||		
+	        (chars[cursor] == 0x0CC6 && chars[cursor + 1] == 0x0CD6) ||		
+	        (chars[cursor] == 0x0CC6 && chars[cursor + 1] == 0x0CC2) ||		
+	        (chars[cursor] == 0x0CCA && chars[cursor + 1] == 0x0CD5))
+		    return cursor + 2;
+	}
+
         if (state < 0) {
             break;
         }
