@@ -31,6 +31,7 @@
 #include "pango-enum-types.h"
 #include "pango-modules.h"
 #include "pango-impl-utils.h"
+#include "modules.h"
 
 typedef struct _PangoModule      PangoModule;
 typedef struct _PangoModuleClass PangoModuleClass;
@@ -518,6 +519,7 @@ static void
 init_modules (void)
 {
   static gboolean init = FALSE;
+  int i;
 
   if (init)
     return;
@@ -527,6 +529,8 @@ init_modules (void)
   /* Make sure that the type system is initialized */
   g_type_init ();
   
+  for (i = 0; _pango_included_lang_modules[i].list; i++)
+    pango_module_register (&_pango_included_lang_modules[i]);
   read_modules ();
 }
 
@@ -680,7 +684,7 @@ append_engines (GSList **engine_list,
  *  handle this script.
  * 
  * Finds engines in the map that handle the given script. The returned
- * lists should be fred with g_slist_free, but the engines in the
+ * lists should be freed with g_slist_free, but the engines in the
  * lists are owned by GLib and will be kept around permanently, so
  * they should not be unref'ed.
  *
