@@ -31,7 +31,7 @@
 /**
  * pango_glyph_item_split:
  * @orig: a #PangoItem
- * @text: text to which positions in @orig apply.
+ * @text: text to which positions in @orig apply
  * @split_index: byte index of position to split item, relative to the start of the item
  * 
  * Modifies @orig to cover only the text after @split_index, and
@@ -43,9 +43,11 @@
  * zero-length item).
  *
  * This function is similar in function to pango_item_split() (and uses
- * it internally)
+ * it internally.)
  * 
- * Return value: new item representing text before @split_index
+ * Return value: the newly allocated item representing text before
+ *               @split_index, which should be freed
+ *               with pango_glyph_item_free().
  *
  * Since: 1.2
  **/
@@ -161,6 +163,7 @@ _pango_glyph_item_iter_next_cluster (PangoGlyphItemIter *iter)
 {
   int glyph_index = iter->end_glyph;
   PangoGlyphString *glyphs = iter->glyph_item->glyphs;
+  int cluster;
   PangoItem *item = iter->glyph_item->item;
 
   if (LTR (iter->glyph_item))
@@ -180,6 +183,7 @@ _pango_glyph_item_iter_next_cluster (PangoGlyphItemIter *iter)
   
   if (LTR (iter->glyph_item))
     {
+      cluster = glyphs->log_clusters[glyph_index];
       while (TRUE)
 	{
 	  glyph_index++;
@@ -191,7 +195,7 @@ _pango_glyph_item_iter_next_cluster (PangoGlyphItemIter *iter)
 	      break;
 	    }
 	  
-	  if (item->offset + glyphs->log_clusters[glyph_index] >= iter->start_index)
+	  if (glyphs->log_clusters[glyph_index] != cluster)
 	    {
 	      iter->end_index = item->offset + glyphs->log_clusters[glyph_index];
 	      iter->end_char += g_utf8_strlen (iter->text + iter->start_index,
@@ -202,6 +206,7 @@ _pango_glyph_item_iter_next_cluster (PangoGlyphItemIter *iter)
     }
   else			/* RTL */
     {
+      cluster = glyphs->log_clusters[glyph_index];
       while (TRUE)
 	{
 	  glyph_index--;
@@ -213,7 +218,7 @@ _pango_glyph_item_iter_next_cluster (PangoGlyphItemIter *iter)
 	      break;
 	    }
 	  
-	  if (item->offset + glyphs->log_clusters[glyph_index] >= iter->start_index)
+	  if (glyphs->log_clusters[glyph_index] != cluster)
 	    {
 	      iter->end_index = item->offset + glyphs->log_clusters[glyph_index];
 	      iter->end_char += g_utf8_strlen (iter->text + iter->start_index,
@@ -233,7 +238,7 @@ _pango_glyph_item_iter_next_cluster (PangoGlyphItemIter *iter)
  * 
  * Moves the iterator to the preceding cluster in the glyph item.
  * 
- * Return value: %TRUE if the iterator was movedn, %FALSE if we were already on the
+ * Return value: %TRUE if the iterator was moved, %FALSE if we were already on the
  *  first cluster.
  **/
 gboolean
@@ -241,6 +246,7 @@ _pango_glyph_item_iter_prev_cluster (PangoGlyphItemIter *iter)
 {
   int glyph_index = iter->start_glyph;
   PangoGlyphString *glyphs = iter->glyph_item->glyphs;
+  int cluster;
   PangoItem *item = iter->glyph_item->item;
     
   if (LTR (iter->glyph_item))
@@ -261,6 +267,7 @@ _pango_glyph_item_iter_prev_cluster (PangoGlyphItemIter *iter)
   
   if (LTR (iter->glyph_item))
     {
+      cluster = glyphs->log_clusters[glyph_index - 1];
       while (TRUE)
 	{
 	  glyph_index--;
@@ -272,7 +279,7 @@ _pango_glyph_item_iter_prev_cluster (PangoGlyphItemIter *iter)
 	      break;
 	    }
 	  
-	  if (item->offset + glyphs->log_clusters[glyph_index] < iter->end_index)
+	  if (glyphs->log_clusters[glyph_index] != cluster)
 	    {
 	      iter->start_index = item->offset + glyphs->log_clusters[glyph_index];
 	      iter->start_char -= g_utf8_strlen (iter->text + iter->start_index,
@@ -283,6 +290,7 @@ _pango_glyph_item_iter_prev_cluster (PangoGlyphItemIter *iter)
     }
   else			/* RTL */
     {
+      cluster = glyphs->log_clusters[glyph_index + 1];
       while (TRUE)
 	{
 	  glyph_index++;
@@ -294,7 +302,7 @@ _pango_glyph_item_iter_prev_cluster (PangoGlyphItemIter *iter)
 	      break;
 	    }
 	  
-	  if (item->offset + glyphs->log_clusters[glyph_index] < iter->start_index)
+	  if (glyphs->log_clusters[glyph_index] != cluster)
 	    {
 	      iter->start_index = item->offset + glyphs->log_clusters[glyph_index];
 	      iter->start_char -= g_utf8_strlen (iter->text + iter->start_index,
@@ -311,14 +319,14 @@ _pango_glyph_item_iter_prev_cluster (PangoGlyphItemIter *iter)
 /**
  * _pango_glyph_item_iter_init_start:
  * @iter: pointer to a #PangoGlyphItemIter structure
- * @glyph_item: the glyph item that the iter points into
+ * @glyph_item: the glyph item that @iter points into
  * @text: text corresponding to the glyph item
  * 
  * Initializes a #PangoGlyphItemIter structure to point to the
  * first cluster in a glyph item.
  * 
  * Return value: %FALSE if there are no clusters in the glyph item;
- *  in this case, the state of the iter is undefined.
+ *  in this case, the state of @iter is undefined.
  **/
 gboolean
 _pango_glyph_item_iter_init_start (PangoGlyphItemIter  *iter,
@@ -343,14 +351,14 @@ _pango_glyph_item_iter_init_start (PangoGlyphItemIter  *iter,
 /**
  * _pango_glyph_item_iter_init_start:
  * @iter: pointer to a #PangoGlyphItemIter structure
- * @glyph_item: the glyph item that the iter points into
+ * @glyph_item: the glyph item that @iter points into
  * @text: text corresponding to the glyph item
  * 
  * Initializes a #PangoGlyphItemIter structure to point to the
  * last cluster in a glyph item.
  * 
  * Return value: %FALSE if there are no clusters in the glyph item;
- *  in this case, the state of the iter is undefined.
+ *  in this case, the state of @iter is undefined.
  **/
 gboolean
 _pango_glyph_item_iter_init_end (PangoGlyphItemIter  *iter,
@@ -389,7 +397,7 @@ append_attrs (PangoGlyphItem *glyph_item,
     g_slist_concat (glyph_item->item->analysis.extra_attrs, attrs);
 }
 
-/* Make a deep copy of a GSlist of PangoAttribute
+/* Make a deep copy of a #GSList of PangoAttribute
  */
 static GSList *
 attr_slist_copy (GSList *attrs)
@@ -445,7 +453,7 @@ split_before_cluster_start (ApplyAttrsState *state)
  *
  * All attributes that start or end inside a cluster are applied
  * to that cluster; for instance, if half of a cluster is underlined
- * and the other-half strikethough, then the cluster will end
+ * and the other-half strikethrough, then the cluster will end
  * up with both underline and strikethrough attributes. In these
  * cases, it may happen that item->extra_attrs for some of the
  * result items can have multiple attributes of the same type.
