@@ -21,7 +21,6 @@
 
 #include <config.h>
 #include <errno.h>
-#include <math.h>
 #include <string.h>
 #include <stdlib.h>
 
@@ -86,10 +85,10 @@ pango_trim_string (const char *str)
 
 /**
  * pango_split_file_list:
- * @str: a G_SEARCHPATH_SEPARATOR separated list of filenames
+ * @str: a %G_SEARCHPATH_SEPARATOR separated list of filenames
  * 
- * Splits a G_SEARCHPATH_SEPARATOR-separated list of files, stripping
- * white space and subsituting ~/ with $HOME/.
+ * Splits a %G_SEARCHPATH_SEPARATOR-separated list of files, stripping
+ * white space and substituting ~/ with $HOME/.
  * 
  * Return value: a list of strings to be freed with g_strfreev()
  **/
@@ -287,7 +286,7 @@ pango_skip_space (const char **pos)
  * of [A-Za-z_] followed by zero or more [A-Za-z_0-9]
  * Leading white space is skipped.
  * 
- * Return value: %FALSE if a parse error occured. 
+ * Return value: %FALSE if a parse error occurred. 
  **/
 gboolean
 pango_scan_word (const char **pos, GString *out)
@@ -330,7 +329,7 @@ pango_scan_word (const char **pos, GString *out)
  * string with '"'. Instead a quoted string, '\"' represents
  * a literal quote. Leading white space outside of quotes is skipped.
  * 
- * Return value: %FALSE if a parse error occured.
+ * Return value: %FALSE if a parse error occurred.
  **/
 gboolean
 pango_scan_string (const char **pos, GString *out)
@@ -413,39 +412,33 @@ pango_scan_string (const char **pos, GString *out)
  * @pos: in/out string position
  * @out: an int into which to write the result
  * 
- * Scans an integer. An integer consists
- * of up to 31 decimal digits. 
+ * Scans an integer. 
  * Leading white space is skipped.
  * 
- * Return value: %FALSE if a parse error occured.
+ * Return value: %FALSE if a parse error occurred.
  **/
 gboolean
 pango_scan_int (const char **pos, int *out)
 {
   unsigned int i = 0;
-  char buf[32];
-  const char *p = *pos;
+  char *end;
+  long temp;
 
-  while (g_ascii_isspace (*p))
-    p++;
-  
-  if (*p < '0' || *p > '9')
-    return FALSE;
-
-  while ((*p >= '0') && (*p <= '9') && i < sizeof(buf))
+  errno = 0;
+  temp = strtol (*pos, &end, 10);
+  if (errno == ERANGE)
     {
-      buf[i] = *p;
-      i++;
-      p++;
+      errno = 0;
+      return FALSE;
+    }
+	
+  *out = (int)temp;
+  if ((long)(*out) != temp)
+    {
+      return FALSE;
     }
 
-  if (i == sizeof(buf))
-    return FALSE;
-  else
-    buf[i] = '\0';
-
-  *out = atoi (buf);
-  *pos = p;
+  *pos = end;
 
   return TRUE;
 }
@@ -929,230 +922,6 @@ pango_parse_stretch (const char   *str,
   return FALSE;
 }
 
-GType
-pango_matrix_get_type (void)
-{
-  static GType our_type = 0;
-  
-  if (our_type == 0)
-    our_type = g_boxed_type_register_static (I_("PangoMatrix"),
-					     (GBoxedCopyFunc) pango_matrix_copy,
-					     (GBoxedFreeFunc) pango_matrix_free);
-
-  return our_type;
-}
-
-/**
- * pango_matrix_copy:
- * @matrix: a #PangoMatrix, can be %NULL
- * 
- * Copies a #PangoMatrix.
- * 
- * Return value: the newly allocated #PangoMatrix, which should
- *               be freed with pango_matrix_free(), or %NULL if
- *               @matrix was %NULL.
- *
- * Since: 1.6
- **/
-PangoMatrix *
-pango_matrix_copy (const PangoMatrix *matrix)
-{
-  PangoMatrix *new_matrix;
-
-  if (matrix)
-    {
-      new_matrix = g_slice_new (PangoMatrix);
-      *new_matrix = *matrix;
-    }
-  else
-    new_matrix = NULL;
-
-  return new_matrix;
-}
-
-/**
- * pango_matrix_free:
- * @matrix: a #PangoMatrix, or %NULL
- * 
- * Free a #PangoMatrix created with pango_matrix_copy().
- * Does nothing if @matrix is %NULL.
- *
- * Since: 1.6
- **/
-void
-pango_matrix_free (PangoMatrix *matrix)
-{
-  if (matrix)
-    g_slice_free (PangoMatrix, matrix);
-}
-
-/**
- * pango_matrix_translate:
- * @matrix: a #PangoMatrix
- * @tx: amount to translate in the X direction
- * @ty: amount to translate in the Y direction
- * 
- * Changes the transformation represented by @matrix to be the
- * transformation given by first translating by (@tx, @ty)
- * then applying the original transformation.
- *
- * Since: 1.6
- **/
-void
-pango_matrix_translate (PangoMatrix *matrix,
-			double       tx,
-			double       ty)
-{
-  g_return_if_fail (matrix != NULL);
-
-  matrix->x0  = matrix->xx * tx + matrix->xy * ty + matrix->x0;
-  matrix->y0  = matrix->yx * tx + matrix->yy * ty + matrix->y0;
-}
-
-/**
- * pango_matrix_scale:
- * @matrix: a #PangoMatrix
- * @scale_x: amount to scale by in X direction
- * @scale_y: amount to scale by in Y direction
- * 
- * Changes the transformation represented by @matrix to be the
- * transformation given by first scaling by @sx in the X direction
- * and @sy in the Y direction then applying the original
- * transformation.
- *
- * Since: 1.6
- **/
-void
-pango_matrix_scale (PangoMatrix *matrix,
-		    double       scale_x,
-		    double       scale_y)
-{
-  g_return_if_fail (matrix != NULL);
-
-  matrix->xx *= scale_x;
-  matrix->xy *= scale_y;
-  matrix->yx *= scale_x;
-  matrix->yy *= scale_y;
-}
-
-/**
- * pango_matrix_rotate:
- * @matrix: a #PangoMatrix
- * @degrees: degrees to rotate counter-clockwise
- * 
- * Changes the transformation represented by @matrix to be the
- * transformation given by first rotating by @degrees degrees
- * counter-clokwise then applying the original transformation.
- *
- * Since: 1.6
- **/
-void
-pango_matrix_rotate (PangoMatrix *matrix,
-		     double       degrees)
-{
-  PangoMatrix tmp;
-  gdouble r, s, c;
-
-  g_return_if_fail (matrix != NULL);
-
-  r = degrees * (G_PI / 180.);
-  s = sin (r);
-  c = cos (r);
-
-  tmp.xx = c;
-  tmp.xy = s;
-  tmp.yx = -s;
-  tmp.yy = c;
-  tmp.x0 = 0;
-  tmp.y0 = 0;
-
-  pango_matrix_concat (matrix, &tmp);
-}
-
-/**
- * pango_matrix_concat:
- * @matrix: a #PangoMatrix
- * @new_matrix: a #PangoMatrix
- * 
- * Changes the transformation represented by @matrix to be the
- * transformation given by first applying transformation
- * given by @new_matrix then applying the original transformation.
- *
- * Since: 1.6
- **/
-void
-pango_matrix_concat (PangoMatrix       *matrix,
-		     const PangoMatrix *new_matrix)
-{
-  PangoMatrix tmp;
-  
-  g_return_if_fail (matrix != NULL);
-
-  tmp = *matrix;
-
-  matrix->xx = tmp.xx * new_matrix->xx + tmp.xy * new_matrix->yx;
-  matrix->xy = tmp.xx * new_matrix->xy + tmp.xy * new_matrix->yy;
-  matrix->yx = tmp.yx * new_matrix->xx + tmp.yy * new_matrix->yx;
-  matrix->yy = tmp.yx * new_matrix->xy + tmp.yy * new_matrix->yy;
-  matrix->x0  = tmp.xx * new_matrix->x0 + tmp.xy * new_matrix->y0 + tmp.x0;
-  matrix->y0  = tmp.yx * new_matrix->y0 + tmp.yy * new_matrix->y0 + tmp.y0;
-}
-
-/**
- * pango_matrix_get_font_scale_factor:
- * @matrix: a #PangoMatrix, may be %NULL
- * 
- * Returns the scale factor of a matrix on the height of the font.
- * That is, the scale factor in the direction perpendicular to the
- * vector that the X coordinate is mapped to.
- *
- * Return value: the scale factor of @matrix on the height of the font,
- * or 1.0 if @matrix is %NULL.
- *
- * Since: 1.12
- **/
-double
-pango_matrix_get_font_scale_factor (const PangoMatrix *matrix)
-{
-/*
- * Based on cairo-matrix.c:_cairo_matrix_compute_scale_factors()
- *
- * Copyright 2005, Keith Packard
- */
-  double det;
-  
-  if (!matrix)
-    return 1.0;
-  
-  det = matrix->xx * matrix->yy - matrix->yx * matrix->xy;
-
-  if (det == 0)
-    {
-      return 0.0;
-    }
-  else
-    {
-      double x = matrix->xx;
-      double y = matrix->yx;
-      double major, minor;
-
-      major = sqrt (x*x + y*y);
-      
-      /*
-       * ignore mirroring
-       */
-      if (det < 0)
-	det = - det;
-      
-      if (major)
-	minor = det / major;
-      else 
-	minor = 0.0;
-
-      return minor;
-    }
-}
-
 static const char canon_map[256] = {
    0,   0,   0,   0,   0,   0,   0,   0,    0,   0,   0,   0,   0,   0,   0,   0, 
    0,   0,   0,   0,   0,   0,   0,   0,    0,   0,   0,   0,   0,   0,   0,   0, 
@@ -1323,7 +1092,7 @@ lang_info_compare (const void *key, const void *val)
 
 /* The following array is supposed to contain enough text to tickle all necessary fonts for each
  * of the languages in the following. Yes, it's pretty lame. Not all of the languages
- * in the following have sufficient text to excercise all the accents for the language, and
+ * in the following have sufficient text to exercise all the accents for the language, and
  * there are obviously many more languages to include as well.
  */
 static const LangInfo lang_texts[] = {
@@ -1397,7 +1166,7 @@ pango_language_get_sample_string (PangoLanguage *language)
  * pango_log2vis_get_embedding_levels:
  * @text:      the text to itemize.
  * @length:    the number of bytes (not characters) to process, or -1
- *             if @text is nul-terminated and the legnth should be calculated.
+ *             if @text is nul-terminated and the length should be calculated.
  * @pbase_dir: input base direction, and output resolved direction.
  *
  * This will return the bidirectional embedding levels of the input paragraph
