@@ -20,7 +20,7 @@
  */
 
 #include <config.h>
-#include <pango/pango-glyph.h>
+#include "pango-glyph.h"
 
 /*
  * NB: The contents of the file implement the exact same algorithm
@@ -31,18 +31,17 @@ static GList *reorder_items_recurse (GList *items, int n_items);
 
 /**
  * pango_reorder_items:
- * @logical_items:  a GList of #PangoItem in logical order.
+ * @logical_items:  a #GList of #PangoItem in logical order.
  *
  * From a list of items in logical order and the associated
  * directional levels, produce a list in visual order.
  * The original list is unmodified.
  *
- * Returns a GList of PangoItem structures in visual order.
+ * Returns a #GList of #PangoItem structures in visual order.
  *
- * (Please mail otaylor@redhat.com if you use this function.
+ * (Please open a bug if you use this function.
  *  It is not a particularly convenient interface, and the code
  *  is duplicated elsewhere in Pango for that reason.)
- *   
  */
 GList *
 pango_reorder_items (GList *logical_items)
